@@ -531,6 +531,10 @@ HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, Hardwa
 HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=1,
                                                               form=HardwareStatsTransformerForm, can_delete_extra=True)
 
+dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
+                                                                     fk_name='parent',
+                                                                     can_delete_extra=True
+                                                                     )
 from django.utils.safestring import mark_safe
 
 
@@ -555,11 +559,11 @@ class DataTransform(forms.ModelForm):
             if self.instance.get_children().count() >= 0:
                 print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))
 
-                dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                     fk_name='parent', form=DataTransform,
-                                                                     extra=self.instance.get_children().count() + 1,
-                                                                     can_delete_extra=True
-                                                                     )
+                # dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
+                #                                                      fk_name='parent', form=DataTransform,
+                #                                                      extra=self.instance.get_children().count() + 1,
+                #                                                      can_delete_extra=True
+                #                                                      )
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     queryset=self.instance.get_children(),
@@ -567,29 +571,27 @@ class DataTransform(forms.ModelForm):
                     files=self.files if self.is_bound else None,
                     initial=[{'parent': self.instance, 'accessory': self.instance.accessory, 'id': str(uuid.uuid4())}]
                 )
-
+                self.nested_data_transformer_form.extra = self.instance.get_children().count() + 1
+                self.nested_data_transformer_form.form = self
             else:
-                dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                     fk_name='parent', form=DataTransform,
-                                                                     extra=0
-                                                                     )
+
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None
                 )
+                self.nested_data_transformer_form.extra = 0
+                self.nested_data_transformer_form.form = self
 
         except Exception as e:
             print(str(e))
-            dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                 fk_name='parent', form=DataTransform,
-                                                                 extra=0
-                                                                 )
             self.nested_data_transformer_form = dataTransformerInlineFormset(
                 instance=self.instance,
                 data=self.data if self.is_bound else None,
                 files=self.files if self.is_bound else None
             )
+            self.nested_data_transformer_form.extra = 0
+            self.nested_data_transformer_form.form = self
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
@@ -643,26 +645,17 @@ class DataTransform(forms.ModelForm):
         print('node')
         print('save: ' + str(self.instance.id))
         root_node = super(DataTransform, self).save(commit)
-        # if hasattr(self,
-        #            'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
-        #     self.nested_constants_form.save(commit)
-        # if hasattr(self,
-        #            'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
-        #     self.nested_hardware_outputs_form.save(commit)
-        # if hasattr(self,
-        #            'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
-        #     self.nested_hardware_stats_form.save(commit)
-        # if hasattr(self,
-        #            'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
-        #     self.nested_data_transformer_form.save(commit)
         if hasattr(self,
                    'nested_constants_form') and self.nested_constants_form is not None:
+            self.nested_constants_form.instance.refresh_from_db()
             if self.nested_constants_form.is_valid():
                 self.nested_constants_form.save(commit)
             else:
                 print(self.nested_constants_form.errors)
         if hasattr(self,
                    'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+            self.nested_hardware_outputs_form.instance.refresh_from_db()
+
             if self.nested_hardware_outputs_form.is_valid():
                 self.nested_hardware_outputs_form.save(commit)
             else:
@@ -675,6 +668,8 @@ class DataTransform(forms.ModelForm):
                 print(self.nested_hardware_stats_form.errors)
         if hasattr(self,
                    'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+            self.nested_data_transformer_form.instance.refresh_from_db()
+
             if self.nested_data_transformer_form.is_valid():
                 self.nested_data_transformer_form.save(commit)
             else:
@@ -708,11 +703,6 @@ class DataTransformRoot(forms.ModelForm):
             if hasattr(self.instance, 'get_children'):
                 print('Adding to root form node: ' + str(self.instance.get_children().count() + 1))
 
-                dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                     fk_name='parent', form=DataTransform,
-                                                                     extra=self.instance.get_children().count() + 1,
-                                                                     can_delete_extra=True
-                                                                     )
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     queryset=self.instance.get_children(),
@@ -720,18 +710,20 @@ class DataTransformRoot(forms.ModelForm):
                     files=self.files if self.is_bound else None,
                     initial=[{'parent': self.instance, 'accessory': self.instance.accessory, 'id': str(uuid.uuid4())}]
                 )
+                self.nested_data_transformer_form.extra = self.instance.get_children().count() + 1
+                self.nested_data_transformer_form.form = self
+
             else:
-                dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                     fk_name='parent', form=DataTransform,
-                                                                     extra=0
-                                                                     )
+
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None
                 )
-        except Exception:
-            print('exception')
+                self.nested_data_transformer_form.extra = 0
+                self.nested_data_transformer_form.form = self
+        except Exception as e:
+            print(str(e))
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
             instance=self.instance,
